# Copyright [2025] [Ian Moore]
# Distributed under the MIT License (license terms are at http://opensource.org/licenses/MIT).
# Email: defipy.devs@gmail.com

from uniswappy.utils.tools.v3 import UniV3Helper
from uniswappy.process.swap import WithdrawSwap
from uniswappy.process.deposit import SwapDeposit

class UniswapScriptHelper():

    
    """ Uniswap script helper functions             
    """       

    def __init__(self):
        pass

    def calc_arb(self, lp, sDel, tkn_x, tkn_y, user_nm, p):  
        swap_dx, swap_dy = sDel.calc(p, 1, 1)
        direction = 0;
    
        if(swap_dx >= 0):
            expected_amount_dep = SwapDeposit().apply(lp, tkn_x, user_nm, abs(swap_dx))
            expected_amount_out = WithdrawSwap().apply(lp, tkn_y, user_nm, abs(swap_dy))
            dep_tkn_x = abs(swap_dx); wd_tkn_x = 0
            wd_tkn_y = abs(swap_dy); dep_tkn_y = 0
            direction = 0
        elif(swap_dy >= 0):
            expected_amount_dep = SwapDeposit().apply(lp, tkn_y, user_nm, abs(swap_dy))
            expected_amount_out = WithdrawSwap().apply(lp, tkn_x, user_nm, abs(swap_dx)) 
            dep_tkn_y = abs(swap_dy); wd_tkn_y = 0
            wd_tkn_x = abs(swap_dx); dep_tkn_x = 0
            direction = 1
        
        dep_tkn_x = UniV3Helper().dec2gwei(dep_tkn_x)
        dep_tkn_y = UniV3Helper().dec2gwei(dep_tkn_y)
        wd_tkn_x = UniV3Helper().dec2gwei(wd_tkn_x)
        wd_tkn_y = UniV3Helper().dec2gwei(wd_tkn_y)
    
        return (dep_tkn_x, dep_tkn_y, wd_tkn_x, wd_tkn_y, direction)
    
    def calc_arb_contract(self, lp, sDel, tkn_x, tkn_y, user_nm, p):  
        swap_dx, swap_dy = sDel.calc(p, 1, 1)
        direction = 0;
    
        if(swap_dx >= 0):
            dep_tkn_x = abs(swap_dx); wd_tkn_x = 0
            wd_tkn_y = abs(swap_dy); dep_tkn_y = 0
            direction = 0
        elif(swap_dy >= 0):
            dep_tkn_y = abs(swap_dy); wd_tkn_y = 0
            wd_tkn_x = abs(swap_dx); dep_tkn_x = 0
            direction = 1
        
        dep_tkn_x = UniV3Helper().dec2gwei(dep_tkn_x)
        dep_tkn_y = UniV3Helper().dec2gwei(dep_tkn_y)
        wd_tkn_x = UniV3Helper().dec2gwei(wd_tkn_x)
        wd_tkn_y = UniV3Helper().dec2gwei(wd_tkn_y)
    
        return (dep_tkn_x, dep_tkn_y, wd_tkn_x, wd_tkn_y, direction)

    def pool_state(self, pool_contract, verbose=False):
        reserves = pool_contract.functions.getReserves().call() 
        lp_amt = pool_contract.functions.totalSupply().call() 
        if(verbose): 
            print(f'TKN0 {UniV3Helper().gwei2dec(reserves[0]):.2f} / TKN1 {UniV3Helper().gwei2dec(reserves[1]):.2f} / Liquidity {UniV3Helper().gwei2dec(lp_amt):.2f}') 
        return (reserves[0], reserves[1], lp_amt)


# Copyright 2023–2025 Ian Moore
# Email: defipy.devs@gmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from uniswappy.process.liquidity import AddLiquidity as UniswapAddLiquidity
from balancerpy.process.liquidity import AddLiquidity as BalancerAddLiquidity
from stableswappy.process.liquidity import AddLiquidity as StableswapAddLiquidity

class AddLiquidity():
    
    """ Add liquidity process             
    """       

    def __init__(self, global_var1 = None):
        self.gvar = global_var1

    def apply(self, lp, v1 = None, v2 = None, v3 = None, v4 = None, v5 = None):
        """ apply

            Add liquidity process

            UniswapAddLiquidity().apply(lp, user_nm, amount0, amount1, lwr_tick = None, upr_tick = None):
            
            BalancerAddLiquidity(Proc.ADDTKN).apply(lp, tkn_in, user_nm, amt_tkn_in)
            
            StableswapAddLiquidity().apply(lp, tkn_in, user_nm, amt_tkn_in)
                     
            Returns
            -------
            out : dictionary
                join output               
        """ 

        if type(lp).__name__ == 'UniswapExchange' or type(lp).__name__ == 'UniswapV3Exchange':
            out = UniswapAddLiquidity().apply(lp, v1, v2, v3, v4, v5)
        elif type(lp).__name__ == 'BalancerExchange':
            out = BalancerAddLiquidity(self.gvar).apply(lp, v1, v2, v3)     
        elif type(lp).__name__ == 'StableswapExchange':
            out = StableswapAddLiquidity().apply(lp, v1, v2, v3)     
        else:
            print('DeFiPy: WRONG EXCHANGE TYPE OR VARIABLES')
             
        return out 



from defipy.erc import *
from defipy.math.basic import *
from defipy.math.interest import *
from defipy.math.interest.ips import *
from defipy.math.interest.ips.aggregate import *
from defipy.math.model import *
from defipy.math.risk import *
from defipy.process import *
from defipy.process.burn import *
from defipy.process.deposit import *
from defipy.process.liquidity import *
from defipy.process.mint import *
from defipy.process.swap import *
from defipy.process.join import *
from defipy.analytics.simulate import *
from defipy.analytics.risk import *
from defipy.utils.interfaces import *
from defipy.utils.data import *
from defipy.utils.client import *
from defipy.utils.client.contract import *
from defipy.utils.tools import *

from uniswappy.cpt.exchg import *
from uniswappy.cpt.factory import *
from uniswappy.cpt.index import *
from uniswappy.cpt.quote import *
from uniswappy.cpt.vault import *
from uniswappy.cpt.wallet import *
from uniswappy.utils.tools.v3 import *

from stableswappy.quote import *
from stableswappy.vault import *
from stableswappy.cst.factory import *
from stableswappy.cst.exchg import *
from stableswappy.utils.data import StableswapExchangeData

from balancerpy.quote import *
from balancerpy.vault import *
from balancerpy.cwpt.factory import *
from balancerpy.cwpt.exchg import *
from balancerpy.enums import *
from balancerpy.utils.data import BalancerExchangeData



  


from setuptools import setup

with open('README.md') as f:
    long_description = f.read()

setup(name='DeFiPy',
      version='1.0.5',
      description='DeFi Analytics with Python',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='http://github.com/defipy-devs/defipy',
      author = "icmoore",
      author_email = "defipy.devs@gmail.com",
      license="Apache-2.0",
      classifiers=[
            "License :: OSI Approved :: Apache Software License",
            "Programming Language :: Python :: 3",
            "Operating System :: OS Independent",
            "Intended Audience :: Developers",
            "Topic :: Software Development :: Libraries :: Python Modules",
            "Topic :: Scientific/Engineering :: Information Analysis",
            "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
      ],
      package_dir = {"defipy": "python/prod"},
      packages=[
          'defipy',
          'defipy.erc',
          'defipy.math.basic',
          'defipy.math.interest',
          'defipy.math.interest.ips',
          'defipy.math.interest.ips.aggregate',
          'defipy.math.model',
          'defipy.math.risk',
          'defipy.process',
          'defipy.process.burn',
          'defipy.process.deposit',
          'defipy.process.liquidity',
          'defipy.process.mint',
          'defipy.process.swap',
          'defipy.process.join',
          'defipy.analytics.simulate',
          'defipy.analytics.risk',
          'defipy.utils.interfaces',
          'defipy.utils.data',
          'defipy.utils.client',
          'defipy.utils.client.contract',
          'defipy.utils.tools',
          'defipy.utils.tools.v3'
      ],
      install_requires=[
        'scipy >= 1.7.3', 
        'bokeh == 3.3.4',  
        'uniswappy == 1.7.3', 
        'stableswappy == 1.0.2',
        'balancerpy == 1.0.3'  
      ],      
      zip_safe=False)



# Copyright [2025] [Ian Moore]
# Distributed under the MIT License (license terms are at http://opensource.org/licenses/MIT).
# Email: defipy.devs@gmail.com

import subprocess

FORGE_CMD = 'forge'
SCRIPT_CMD = 'script'
SILENT_OPT = '--silent'
SKIP_SIM_OPT = '--skip-simulation'
SIG_OPT = '--sig'
SIG_OPT = '--sig'
RPC_OPT = '--rpc-url'
BROADCAST_OPT = '--broadcast'

class ExecuteScript():

    def __init__(self, test_directory):
        self.test_directory = test_directory
        pass

    def apply(self, script_path, rpc_url, args = [], verbose = True, skip_sim = True):
        shell_cmd = self._exe_cmd(script_path, rpc_url, args=args, verbose=verbose, skip_sim=skip_sim)
        p = subprocess.Popen(shell_cmd, cwd=self.test_directory)
        p.wait()
          
    def _exe_cmd(self, script_path, rpc_url, verbose = True, skip_sim = True, args = []):
        silent_cmd = SILENT_OPT if not verbose else ''
        skip_sim_cmd = SKIP_SIM_OPT if skip_sim else ''
        deploy_sig = [] if len(args) == 0 else [SIG_OPT]
        deploy_arg_type = [] if len(args) == 0 else ['run('+','.join('uint256' for e in args)+')']
        deploy_args = [] if len(args) == 0 else [str(e) for e in args]
    
        shell_cmd1 = [FORGE_CMD, SCRIPT_CMD, script_path]
        shell_cmd2 = deploy_args + deploy_sig + deploy_arg_type
        shell_cmd3 = [ '--ignored-error-codes', '2072', 
                    '--ignored-error-codes', '6321', 
                    '--ignored-error-codes', '5667', 
                    '--ignored-error-codes', '5574', 
                     '--ignored-error-codes', '2018']
        shell_cmd4 = [RPC_OPT, rpc_url, BROADCAST_OPT,  silent_cmd, skip_sim_cmd]
        shell_cmd = shell_cmd1 + shell_cmd2 + shell_cmd3 + shell_cmd4 
        return shell_cmd